import type { FormKitFrameworkContext, FormKitTypeDefinition } from '@formkit/core';
import type { FormKitInputs } from '@formkit/inputs';
import { createSection, label, outer } from '@formkit/inputs';
import PrimevueSelect, { type SelectEmitsOptions } from 'primevue/select';
import { defineComponent, markRaw } from 'vue';
import { floatLabel } from '../sections/floatLabel';
import { messages } from '../sections/messages';

type PrimevueSelectListeners = {
  // 'onFocus': SelectEmitsOptions['focus'];
  'onUpdate:modelValue': SelectEmitsOptions['update:modelValue'];
  // 'onChange': SelectEmitsOptions['change'];
  'onBlur': SelectEmitsOptions['blur'];
};

const PSelectComp = defineComponent(
  (vueProps: { context: FormKitFrameworkContext }) => {
    const formkitContext = vueProps.context;
    const listeners: PrimevueSelectListeners = {
      'onUpdate:modelValue': (value: any) => {
        formkitContext.node.input(value);
      },
      'onBlur': async e => {
        setTimeout(() => formkitContext.handlers.blur.call(undefined, e as never), 20);
      },
      /* https://formkit.com/essentials/inputs#debouncing
       * The default debounce delay is 20 milliseconds and can be adjusted with the delay prop or config option.
       * 延迟 prop 的默认值是 20 毫秒。然而，group 和 list 输入默认使用 0 毫秒，以防止防抖延迟在每个深度级别“累积”。
       * 不过好像不是这个造成的`blur`比`change`先触发。
       */
    };

    return () => {
      return (
        <PrimevueSelect
          labelId={formkitContext.id}
          fluid
          invalid={formkitContext.state.invalid}
          disabled={!!formkitContext.disabled}
          options={formkitContext.options}
          modelValue={formkitContext._value}
          {...listeners}
        />
      );
    };
  },
  // https://cn.vuejs.org/api/general#definecomponent
  // 目前仍然需要手动声明运行时的 props
  // 在将来，我们计划提供一个 Babel 插件，自动推断并注入运行时 props (就像在单文件组件中的 defineProps 一样)，以便省略运行时 props 的声明。
  {
    props: ['context'],
  },
);

const input = createSection('input', () => ({
  $cmp: markRaw(PSelectComp) as never,
  bind: '$attrs',
  props: {
    context: '$node.context',
  },
}));

export const PSelect: FormKitTypeDefinition = {
  type: 'input',
  schema: outer(
    floatLabel(
      input(), //
      label('$label'),
    ),
    messages(),
  ),
  props: ['options'],
  // schemaMemoKey: 'nnvujvlf2xr', // Math.random().toString(36).substring(2, 15)
};

declare module '@formkit/inputs' {
  // https://formkit.com/essentials/custom-inputs#typescript-support
  interface FormKitInputProps<Props extends FormKitInputs<Props>> {
    PSelect: {
      type: 'PSelect';
      options: any;
    };
  }
}
